<?php
/*
 *
 * 

Copyright 2010 Daniel Morante. All rights reserved.

    Redistribution and use in source and binary forms, with or without modification, are
    permitted provided that the following conditions are met:

       1. Redistributions of source code must retain the above copyright notice, this list of
          conditions and the following disclaimer.

       2. Redistributions in binary form must reproduce the above copyright notice, this list
          of conditions and the following disclaimer in the documentation and/or other materials
          provided with the distribution.

    THIS SOFTWARE IS PROVIDED BY Daniel Morante ``AS IS'' AND ANY EXPRESS OR IMPLIED
    WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
    FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL Daniel Morante OR
    CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
    CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
    SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
    ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
    ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    The views and conclusions contained in the software and documentation are those of the
    authors and should not be interpreted as representing official policies, either expressed
    or implied, of Daniel Morante.
 */

/**
 * Description of autoConfig
 *
 * @author Daniel Morante <daniel@morante.net>
 */
class AutoConfig {
    private $DOC = null;
    private $ROOT = null;

    private $HOST = 'mail.exemple.fr';
    private $SOCK = 'SSL';
    private $AUTH = 'password-cleartext';
    private $EMAIL = 'contact@exemple.fr';
    private $DOMAIN = 'exemple.fr';

    private $URL = 'https://doc.ycharbi.fr/index.php/Serveur_de_courriels';

    //put your code here
    public function __construct() {
        //Apply defined constants or use defaults
        if(defined('AUTOCONFIG_DEFAULT_MAIL_HOST')){$this->HOST = AUTOCONFIG_DEFAULT_MAIL_HOST;};
        if(defined('AUTOCONFIG_DEFAULT_DOMAIN')){$this->DOMAIN = AUTOCONFIG_DEFAULT_DOMAIN;};
        if(defined('AUTOCONFIG_DEFAULT_ADDRESS')){$this->EMAIL = AUTOCONFIG_DEFAULT_ADDRESS;};

        //create the xml document
        $this->DOC = new DOMDocument();
        $this->getEmailAddress();
        $this->getDomain();
        $this->getHost();
        //create the root element
        $this->ROOT = $this->DOC->createElement("clientConfig");
        $attr = $this->DOC->createAttribute("version");
        $attr->appendChild($this->DOC->createTextNode("1.1"));
        $this->ROOT->appendChild($attr);

        $this->DOC->appendChild($this->ROOT);                
        $this->ROOT->appendChild($this->makeXML());
    }

    private function getEmailAddress(){
        if(isset($_GET['emailaddress']) && !empty($_GET['emailaddress'])){
            $this->EMAIL = strtolower($_GET['emailaddress']);
        }
        else {
            $this->EMAIL = "%EMAILADDRESS%";
        }
    }
    
    private function getDomain(){
        if($this->EMAIL != "%EMAILADDRESS%"){
            //Split the email address into it's USER and DOMAIN components
            list($userName, $mailDomain) = explode("@", $this->EMAIL);
            $this->DOMAIN = strtolower($mailDomain);
        }
        else{
            $this->DOMAIN = "%EMAILDOMAIN%";
        }
    }

    private function getHost(){
        $this->HOST = "mail." . $this->DOMAIN;
    }


    private function makeXML(){
        //Providor ID Used Internally
        $emailProvider = $this->DOC->createElement("emailProvider");
            $attr = $this->DOC->createAttribute("id");
            $attr->appendChild($this->DOC->createTextNode("mail.exemple.fr"));
            $emailProvider->appendChild($attr);

        //The General Account Info such as displayname, domian, and shortname
        $domain = $this->DOC->createElement("domain", $this->DOMAIN);
        $displayName = $this->DOC->createElement("displayName", "%EMAILADDRESS%");
        $displayShortName = $this->DOC->createElement("displayShortName", str_replace(".", "_", $this->DOMAIN));        
        $emailProvider->appendChild($domain);
        $emailProvider->appendChild($displayName);        
        $emailProvider->appendChild($displayShortName);                

        //-----Incomming------
        //IMAP
        $incomingServer = $this->DOC->createElement("incomingServer");
        $type_attrib = $this->DOC->createAttribute("type");
        $type_attrib->appendChild($this->DOC->createTextNode("imap"));
        $incomingServer->appendChild($type_attrib);        
        $this->makeServer($incomingServer, "993");
        $emailProvider->appendChild($incomingServer);

        //POP3
#        $incomingServer = $this->DOC->createElement("incomingServer");
#        $type_attrib = $this->DOC->createAttribute("type");
#        $type_attrib->appendChild($this->DOC->createTextNode("pop3"));
#        $incomingServer->appendChild($type_attrib);
#        $this->makeServer($incomingServer, "110");
#        $emailProvider->appendChild($incomingServer);


        //----Outgoing----
        //Message Submission
        $outgoingServer = $this->DOC->createElement("outgoingServer");
        $type_attrib = $this->DOC->createAttribute("type");
        $type_attrib->appendChild($this->DOC->createTextNode("smtp"));
        $outgoingServer->appendChild($type_attrib);
        $this->makeServer($outgoingServer, "465");
        $emailProvider->appendChild($outgoingServer);

        //SMTP
#        $outgoingServer = $this->DOC->createElement("outgoingServer");
#        $type_attrib = $this->DOC->createAttribute("type");
#        $type_attrib->appendChild($this->DOC->createTextNode("smtp"));
#        $outgoingServer->appendChild($type_attrib);  
#        $this->makeServer($outgoingServer, "465");
#        $emailProvider->appendChild($outgoingServer);

        //Provide Instructions and Link
        $emailProvider->appendChild($this->makeDesc());

        return $emailProvider;
    }
    
    private function makeDesc(){
        //Description of Support Link
        $desc = $this->DOC->createElement("descr", "Serveur de messagerie de exemple.fr");
        $attr = $this->DOC->createAttribute("lang");
        $attr->appendChild($this->DOC->createTextNode("fr"));
        $desc->appendChild($attr);

        //Support Link
        $documentation = $this->DOC->createElement("documentation");
        $attr = $this->DOC->createAttribute("url");
        $attr->appendChild($this->DOC->createTextNode($this->URL));
        $documentation->appendChild($attr);

        //Join them together and we are done
        $documentation->appendChild($desc);
        return $documentation;
    }

    private function makeServer($parent, $port){
        //Self Explanitory, makes a server for the given parent node on the given port
        //Supports only plain authentication for now.
        $hostname = $this->DOC->createElement("hostname", $this->HOST);
        $port = $this->DOC->createElement("port", $port);
        $socketType = $this->DOC->createElement("socketType", $this->SOCK);
        $authentication = $this->DOC->createElement("authentication", $this->AUTH);
        $username = $this->DOC->createElement("username", $this->EMAIL);

        $parent->appendChild($hostname);
        $parent->appendChild($port);
        $parent->appendChild($socketType);
        $parent->appendChild($authentication);
        $parent->appendChild($username);
    }

    public function getDoc(){
        //make the output pretty
        $this->DOC->formatOutput = true;
        return $this->DOC->saveXML();
    }
}
?>
